@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import, unicode_literals, division
+import six
 import logging
+from collections import Sequence
 from armet import http
 from armet.resources.resource import base
 
@@ -81,34 +83,6 @@ class ManagedResource(base.Resource):
     #! requested. None if a list is being requested.
     slug = None
 
-    # def __new__(cls, request, response):
-    #     # Parse any arguments out of the path.
-    #     arguments = cls.parse(request.path)
-
-    #     # Traverse down the path and determine to resource we're actually
-    #     # accessing.
-    #     cls = cls.traverse(arguments)
-
-    #     # Actually construct the resource and return the instance.
-    #     obj = super(ManagedResource, cls).__new__(cls)
-
-    #     # Update our instance dictionary with the arugments from `parse`.
-    #     # Note that this adds the 'directives', 'query', 'slug', 'path', and
-    #     # 'extensions' attributes.
-    #     obj.__dict__.update(**arguments)
-
-    #     # Return our constructed instance.
-    #     return obj
-
-    #! Precompiled regular expression used to parse out the path.
-    # _parse_pattern = re.compile(
-    #     r'^'
-    #     r'(?:\:(?P<directives>[^/\(\)]*))?'
-    #     r'(?:\((?P<query>[^/]*)\))?'
-    #     r'(?:/(?P<slug>[^/]+?))?'
-    #     r'(?:/(?P<path>.+?))??'
-    #     r'(?:\.(?P<extensions>[^/]+?))??/??$')
-
     @classmethod
     def parse(cls, path):
         result = super(ManagedResource, cls).parse(path)
@@ -130,29 +104,6 @@ class ManagedResource(base.Resource):
         # Return what we got.
         return result
 
-    # @classmethod
-    # def parse(cls, path=''):
-    #     """Parses out parameters and separates them out of the path."""
-    #     # Apply the compiled regex.
-    #     arguments = re.match(cls._parse_pattern, path).groupdict()
-
-    #     # Explode the list arguments; they should be represented as
-    #     # empty arrays and not None.
-    #     if arguments['extensions']:
-    #         arguments['extensions'] = arguments['extensions'].split('.')
-
-    #     else:
-    #         arguments['extensions'] = []
-
-    #     if arguments['directives']:
-    #         arguments['directives'] = arguments['directives'].split(':')
-
-    #     else:
-    #         arguments['directives'] = []
-
-    #     # Return the arguments
-    #     return arguments
-
     @property
     def allowed_operations(self):
         """Retrieves the allowed operations for this request."""
@@ -166,24 +117,25 @@ class ManagedResource(base.Resource):
         if not set(args).issubset(self.allowed_operations):
             raise http.exceptions.Forbidden()
 
-    def make_response(self, data, status=http.client.OK):
+    def make_response(self, data=None, status=http.client.OK):
         """Fills the response object from the passed data."""
-        # Prepare the data for transmission.
-        data = self.prepare(data)
+        if data is not None:
+            # Prepare the data for transmission.
+            data = self.prepare(data)
 
-        # Encode the data using a desired encoder.
-        self.response.write(data, serialize=True)
+            # Encode the data using a desired encoder.
+            self.response.write(data, serialize=True)
 
         # Make sure that the status code is set.
         self.response.status = status
 
     def prepare(self, data):
-        """Prepares the data for encoding."""
         if data is None:
             # No data; return nothing.
             return None
 
-        if self.slug is None:
+        if (not isinstance(data, six.string_types)
+                and isinstance(data, Sequence)):
             # Attempt to prepare each item of the iterable (as long as
             # we're not a string or some sort of mapping).
             for index, value in enumerate(data):
@@ -195,7 +147,6 @@ class ManagedResource(base.Resource):
         return self.item_prepare(data)
 
     def item_prepare(self, item):
-        """Prepares a single item for encoding."""
         # Initialize the object that hold the resultant item.
         obj = {}
 
@@ -203,15 +154,47 @@ class ManagedResource(base.Resource):
         for name, attribute in self.attributes.items():
             if attribute.include:
                 # Run the attribute through its prepare cycle.
-                obj[name] = self.preparers[name](
-                    # Micro preparation cycle on the attribute object.
-                    self, item, attribute.prepare(
+                obj[name] = attribute.prepare(
+                    # Optional preparation cycle on the resource.
+                    self.preparers[name](self, item,
                         # Retrieves the value from the object.
                         attribute.get(item)))
 
         # Return the resultant object.
         return obj
 
+    def clean(self, data):
+        if not data:
+            # If no data; return an empty object.
+            return {}
+
+        if (not isinstance(data, six.string_types)
+                and isinstance(data, Sequence)):
+            # Attempt to clean each item of the iterable (as long as
+            # we're not a string or some sort of mapping).
+            for index, value in enumerate(data):
+                data[index] = self.item_clean(data[index])
+
+            return data
+
+        # Clean just the singular value and return.
+        return self.item_clean(data)
+
+    def item_clean(self, item):
+        # Initialize the object that hold the resultant item.
+        obj = {}
+
+        # Iterate through the attributes and build the object from the item.
+        for name, attribute in self.attributes.items():
+            if attribute.include:
+                # Run the attribute through its prepare cycle.
+                obj[name] = self.cleaners[name](
+                    # Micro preparation cycle on the attribute object.
+                    self, attribute.clean(item.get(name)))
+
+        # Return the resultant object.
+        return obj
+
     @property
     def http_allowed_methods(self):
         if self.slug is None:
@@ -252,54 +235,94 @@ class ManagedResource(base.Resource):
         # Build the response object.
         self.make_response(items)
 
-    # def read(self):
-    #     """Retrieves data to be displayed; called via GET.
+    def post(self, request, response):
+        """Processes a `POST` request."""
+        if self.slug is not None:
+            # Don't know what to do an item access.
+            raise http.exceptions.NotImplemented()
 
-    #     @returns
-    #         Either a single object or an iterable of objects to be encoded
-    #         and returned to the client.
-    #     """
-    #     # There is no default behavior.
-    #     raise http.exceptions.NotImplemented()
+        import ipdb; ipdb.set_trace()
 
-    # def create(self, data):
-    #     """Creates the object that is being requested; via POST or PUT.
+        # Ensure we're allowed to create a resource.
+        self.assert_operations('create')
 
-    #     @param[in] data
-    #         The data to create the object with.
+        # Deserialize and clean the incoming object.
+        data = self.clean(self.request.read(deserialize=True))
 
-    #     @returns
-    #         The object that has been created; or, None, to indicate that no
-    #         object was created.
-    #     """
-    #     # There is no default behavior.
-    #     raise http.exceptions.NotImplemented()
+        try:
+            # Delegate to `create` to create the item.
+            item = self.create(data)
 
-    # def update(self, obj, data):
-    #     """Updates the object that is being requested; via PATCH or PUT.
+        except AttributeError:
+            # No read method defined.
+            raise http.exceptions.NotImplemented()
 
-    #     @param[in] obj
-    #         The objects represented by the current request; the results of
-    #         invoking `self.read()`.
+        # Build the response object.
+        self.make_response(item, status=http.client.CREATED)
 
-    #     @param[in] data
-    #         The data to update the object with.
+    def put(self, request, response):
+        """Processes a `PUT` request."""
+        if self.slug is None:
+            # Mass-PUT is not implemented.
+            raise http.exceptions.NotImplemented()
 
-    #     @returns
-    #         The object that has been updated.
-    #     """
-    #     # There is no default behavior.
-    #     raise http.exceptions.NotImplemented()
+        try:
+            # Check if the resource exists.
+            target = self.read()
 
-    # def destroy(self, obj):
-    #     """Destroy the passed object (or objects).
+        except AttributeError:
+            # No read method defined.
+            raise http.exceptions.NotImplemented()
 
-    #     @param[in] obj
-    #         The objects represented by the current request; the results of
-    #         invoking `self.read()`.
+        # Deserialize and clean the incoming object.
+        data = self.clean(self.request.read(deserialize=True))
 
-    #     @returns
-    #         Nothing.
-    #     """
-    #     # There is no default behavior.
-    #     raise http.exceptions.NotImplemented()
+        if target is not None:
+            # Ensure we're allowed to update the resource.
+            self.assert_operations('update')
+
+            try:
+                # Delegate to `update` to create the item.
+                self.update(target, data)
+
+            except AttributeError:
+                # No read method defined.
+                raise http.exceptions.NotImplemented()
+
+            # Build the response object.
+            self.make_response(target, status=http.client.OK)
+
+        else:
+            # Ensure we're allowed to create the resource.
+            self.assert_operations('create')
+
+            try:
+                # Delegate to `create` to create the item.
+                target = self.create(data)
+
+            except AttributeError:
+                # No read method defined.
+                raise http.exceptions.NotImplemented()
+
+            # Build the response object.
+            self.make_response(target, status=http.client.CREATED)
+
+    def delete(self, request, response):
+        """Processes a `DELETE` request."""
+        if self.slug is None:
+            # Mass-DELETE is not implemented.
+            raise http.exceptions.NotImplemented()
+
+        # Ensure we're allowed to destroy a resource.
+        self.assert_operations('destroy')
+
+        try:
+            # Delegate to `destroy` to destroy the item.
+            self.destroy()
+
+        except AttributeError:
+            # No read method defined.
+            raise http.exceptions.NotImplemented()
+
+        # Build the response object.
+        self.make_response(status=http.client.NO_CONTENT)
